--
-- WardrobeMalfunction
-- 
-- Copyright (c) 2020 Dana Basken
--

local VERSION="9.0.5.1"

WardrobeMalfunction = LibStub("AceAddon-3.0"):NewAddon(ADDON_NAME, "AceConsole-3.0", "AceTimer-3.0", "AceEvent-3.0", "AceComm-3.0")

print(PrefixString(ADDON_NAME, ("v%s"):format(VERSION), PREFIX_COLOR))

local OPTIONS = buildOptions(ADDON_NAME, WardrobeMalfunction)

local AceGUI = LibStub("AceGUI-3.0") -- needed for AceConfigDialog
local AceEvent = LibStub("AceEvent-3.0")
local AceTimer = LibStub("AceTimer-3.0")

function WardrobeMalfunction:OnInitialize()
  self.db = LibStub("AceDB-3.0"):New("WardrobeMalfunctionDB", DEFAULTS, true)

  self.transmogifierState = StateMachine:Create('TRANSMOGRIFIER')
  self.malfunctionState = StateMachine:Create('MALFUNCTION', 'false')
  self.specializationState = StateMachine:Create("SPECIALIZATION")

  LibStub("AceConfig-3.0"):RegisterOptionsTable(ADDON_NAME, OPTIONS)
  LibStub("AceConfigDialog-3.0"):AddToBlizOptions(ADDON_NAME)

  self:CreateFlasher()

  WardrobeMalfunction:RegisterMessage("STATE_CHANGE_TRANSMOGRIFIER")
  WardrobeMalfunction:RegisterMessage("STATE_CHANGE_MALFUNCTION")
  WardrobeMalfunction:RegisterMessage("STATE_CHANGE_SPECIALIZATION")

  WardrobeMalfunction:RegisterEvent("TRANSMOGRIFY_OPEN")
  WardrobeMalfunction:RegisterEvent("TRANSMOGRIFY_SUCCESS")
  WardrobeMalfunction:RegisterEvent("TRANSMOGRIFY_CLOSE")
  WardrobeMalfunction:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
  WardrobeMalfunction:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")

  WardrobeMalfunction:RegisterChatCommand(CHAT_COMMAND_NAME, "HandleChatCommand")
end

function WardrobeMalfunction:OnEnable()
  self.specializationState:SetState(self:GetSpec())
  self:DelayedCheckForWardrobeMalfunction()
end

function WardrobeMalfunction:STATE_CHANGE_TRANSMOGRIFIER(event, state)
  -- print(("%s: %s"):format(event, state))
end

function WardrobeMalfunction:STATE_CHANGE_MALFUNCTION(event, state)
  -- print(("%s: %s"):format(event, state))
  if state == 'true' then
    self:ShowRaidWarning()
    self:PlayAlertSound()
    self:FlashBorder()
    self:AddonPrint("Wardrobe malfunction detected!")
  else
    self:StopFlashing()
  end
end

function WardrobeMalfunction:STATE_CHANGE_SPECIALIZATION(event, state)
  -- print(("%s: %s"):format(event, state))
  self:ShowTransmogLoadedMessage()
end

function WardrobeMalfunction:ShowRaidWarning()
  RaidNotice_AddMessage(RaidWarningFrame, "WARDROBE MALFUNCTION", ChatTypeInfo["RAID_WARNING"])
end

function WardrobeMalfunction:PlayAlertSound()
  if self.db.profile.playSound then
    PlaySoundFile(SOUNDS[self.db.profile.sound], "Master")
  end
end

function WardrobeMalfunction:FlashBorder()
  if self.db.profile.flashBorder then
    self:StartFlashing()
  end
end

function WardrobeMalfunction:StartFlashing()
  if self.animationGroup:IsPlaying() then self:StopFlashing() end
  self.frame.texture:SetVertexColor(self.db.profile.borderColor.r, self.db.profile.borderColor.g, self.db.profile.borderColor.b);
  self.animationGroup:Play()
  self.frame:Show()
  if self.flashTicker then self.flashTicker:Cancel() end
  self.flashTicker = C_Timer.NewTicker(4, function() self:StopFlashing() end, 0)
end

function WardrobeMalfunction:StopFlashing()
  if self.flashTicker then self.flashTicker:Cancel() end
  self.flashTicker = nil
  self.frame:Hide()
  self.animationGroup:Stop()
end

function WardrobeMalfunction:TRANSMOGRIFY_OPEN(...)
end

function WardrobeMalfunction:TRANSMOGRIFY_SUCCESS(...)
  if self.db.profile.autoSave then
    self:SaveCurrentSpecOutfit()
  end
end

function WardrobeMalfunction:ShowTransmogSavedMessage(spec)
  print(PrefixString(ADDON_NAME, ("Transmog saved for spec: %s"):format(spec), PREFIX_COLOR))
end

function WardrobeMalfunction:TRANSMOGRIFY_CLOSE(...)
  self:DelayedCheckForWardrobeMalfunction()
end

function WardrobeMalfunction:PLAYER_EQUIPMENT_CHANGED(...)
  self.specializationState:SetState(self:GetSpec())
  self:DelayedCheckForWardrobeMalfunction()
end

function WardrobeMalfunction:PLAYER_SPECIALIZATION_CHANGED(...)
  self.specializationState:SetState(self:GetSpec())
  self:DelayedCheckForWardrobeMalfunction()
end

function WardrobeMalfunction:ShowTransmogLoadedMessage()
  if self.loadedMessageTicker then self.loadedMessageTicker:Cancel() end
  self.loadedMessageTicker = C_Timer.NewTicker(0.25, function()
    self.loadedMessageTicker = nil
    local spec = self:GetSpec()
    if self.db.profile.outfits[spec] then
      print(PrefixString(ADDON_NAME, ("Transmog loaded for spec: %s"):format(spec), PREFIX_COLOR))
    end
  end, 0)
end

function WardrobeMalfunction:DelayedCheckForWardrobeMalfunction()
  if not self.waitingToCheckForWardrobeMalfunction then
    self.waitingToCheckForWardrobeMalfunction = true
    if self.checkTicker then self.checkTicker:Cancel() end
    self.checkTicker = C_Timer.NewTicker(1, function()
      self.checkTicker = nil
      self:CheckForWardrobeMalfunction()
      self.waitingToCheckForWardrobeMalfunction = false
    end, 0)
  end
end

function WardrobeMalfunction:CreateFlasher()
  self.frame = CreateFrame("Frame", nil, UIParent, BackdropTemplateMixin and "BackdropTemplate")
  self.frame:SetFrameStrata("BACKGROUND");
  self.frame:SetClampedToScreen(true)
  self.frame:SetAllPoints(UIParent)

  local texture = self.frame:CreateTexture(nil,"BACKGROUND");
  texture:SetTexture("Interface\\AddOns\\WardrobeMalfunction\\images\\border.tga");
  texture:SetBlendMode("ADD");
  texture:SetVertexColor(self.db.profile.borderColor.r, self.db.profile.borderColor.g, self.db.profile.borderColor.b);
  texture:SetAllPoints(self.frame);
  self.frame.texture = texture;

  self.frame:SetPoint("CENTER",0,0);
  self.frame:Hide();

  self.animationGroup = self.frame:CreateAnimationGroup()
  self.animationGroup:SetLooping("BOUNCE")

  local animation = self.animationGroup:CreateAnimation("Alpha")
  animation:SetDuration(0.75)
  animation:SetFromAlpha(0.1)
  animation:SetToAlpha(0.9)
end

function WardrobeMalfunction:ShouldCheckForMalfunction()
  if self.transmogifierState.state == "opened" then return false end
  local level = UnitLevel("player")
  if not self.db.profile.checkLeveling and level < MAX_LEVEL then return false end
  return true
end

function WardrobeMalfunction:CheckForWardrobeMalfunction()
  if self:ShouldCheckForMalfunction() then
    local outfit = self:GetCurrentOutfit()
    local spec = self:GetSpec()
    if not self.db.profile.outfits[spec] or outfit == self.db.profile.outfits[spec] then
      self.malfunctionState:SetState('false')
    else
      self.malfunctionState:SetState('true')
    end
  else
    print('not checking transmog')
  end
end

function WardrobeMalfunction:GetSpec()
  local currentSpec = GetSpecialization()
  local currentSpecName = currentSpec and select(2, GetSpecializationInfo(currentSpec)) or "None"
  return currentSpecName
end

function WardrobeMalfunction:GetCurrentOutfit()
  local outfit = ""
  for k, transmog in pairs(TRANSMOG_SLOTS) do
    local transmogLocation = transmog.location
    local isTransmogrified, hasPending, isPendingCollected, canTransmogrify, cannotTransmogrifyReason, hasUndo, isHideVisual, texture = C_Transmog.GetSlotInfo(transmogLocation)
    local equipmentId
    if texture then equipmentId = texture else equipmentId = 0 end
    if outfit ~= "" then outfit = outfit .. ";" end
    outfit = outfit .. transmogLocation:GetSlotName() .. ":" .. equipmentId
  end
  return outfit
end

function WardrobeMalfunction:SaveCurrentSpecOutfit()
  local spec = self:GetSpec()
  self.db.profile.outfits[spec] = self:GetCurrentOutfit()
  self.malfunctionState:SetState('false')
  if self.chatMessageTimer then self:CancelTimer(self.chatMessageTimer) end
  self.chatMessageTimer = self:ScheduleTimer("ShowTransmogSavedMessage", 1, spec)
end

function WardrobeMalfunction:ClearAllOutfits()
  local count = 0
  for k, v in pairs(self.db.profile.outfits) do
    print(PrefixString(ADDON_NAME, ("Clearing transmog for spec: %s"):format(k), PREFIX_COLOR))
    self.db.profile.outfits[k] = nil
    count = count + 1
  end
  if count == 0 then print(PrefixString(ADDON_NAME, "No transmogs found", PREFIX_COLOR)) end
end

function WardrobeMalfunction:GetOption(info)
  -- print(ObjectToString(info.option))
  local property = info.option.arg
  return self.db.profile[property]
end

function WardrobeMalfunction:SetOption(info, value)
  -- print(ObjectToString(info.option), value)
  local property = info.option.arg
  self.db.profile[property] = value
end

function WardrobeMalfunction:SetBorderColor(info, r, g, b, a)
  self.db.profile.borderColor.r = r
  self.db.profile.borderColor.g = g
  self.db.profile.borderColor.b = b
  self:StartFlashing()
end

function WardrobeMalfunction:GetBorderColor(info)
  return self.db.profile.borderColor.r, self.db.profile.borderColor.g, self.db.profile.borderColor.b, 1.0
end

function WardrobeMalfunction:AddonPrint(format, ...)
  print(PrefixString(ADDON_NAME, string.format(format, ...), PREFIX_COLOR))
end

function WardrobeMalfunction:ShowOutfits()
  local specs = ""
  for spec, outfit in pairs(self.db.profile.outfits) do
    if specs ~= "" then specs = specs .. ", " end
    specs = specs .. spec
  end
  if specs == "" then
    self:AddonPrint("No transmogs saved")
  else
    self:AddonPrint("Transmogs saved for these specs: %s", specs)
  end
end

function WardrobeMalfunction:HandleChatCommand(input)
  if not input or input:trim() == "" then
    LibStub("AceConfigDialog-3.0"):Open("WardrobeMalfunction")
  else
    if input == "help" then
      self:AddonPrint("%s", "Commands:")
      self:AddonPrint("  %s", "/wama clear - Clear saved transmogs")
      self:AddonPrint("  %s", "/wama list - List saved transmogs")
      self:AddonPrint("  %s", "/wama save - Save current spec's transmog")
    else
      if input == "clear" then
        self:ClearAllOutfits()
      elseif input == "save" then
        self:SaveCurrentSpecOutfit()
      elseif input == "list" then
        self:ShowOutfits()
      else
        self:AddonPrint("unknown command: '%s', try '/%s help'", input, CHAT_COMMAND_NAME)
      end
    end
    -- LibStub("AceConfigCmd-3.0").HandleCommand(WardrobeMalfunction, CHAT_COMMAND_NAME, "WardrobeMalfunction", input)
  end
end
